/*
    BDate - Blacky Date

    Copyright (C) Andreas Hofmeier

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


    Name:     Andreas Hofmeier

    Adresse:  Stralsunder Strae 20
              28239 Bremen
              Germany

    Telefon:  +49-421-613793

    eMail:    An-H@An-H.de
              hofmeier@inbremen.de
              an-h@gmx.de

    Homepage: http://www.An-H.de/
	      hofmeier.home.pages.de
              http://www.LGut.uni-bremen.de/anh/

*/

#include <stdio.h>
#include <unistd.h>
#include <time.h>
#include <sys/time.h>
#include <sys/types.h>

#include <sys/stat.h>



char blacky_ver_info[] = "BDate v0.01b (c) Copyright 2001-2002 Andreas Hofmeier";


char *tage[] = { "So", "Mo", "Di", "Mi", "Do", "Fr", "Sa" };
char *mon3[] = { "Jan", "Feb", "Mar", "Apr", "Mai", "Jun", "Jul", "Aug", "Sep", "Okt", "Nov", "Dez"};
char mont[]  = {  31,   30,    31,    30,    31,    30,    31,    31,    30,     31,   30,    31};

unsigned long timedatenow;
static struct tm *tminfo;
int year, mon, day, dow, stu, min, doy, week, sec;
int xday = -1;
fd_set rfds;
struct timeval tv;

gettimedate(rtime)
  unsigned long rtime;
{
  if (rtime != 0) {
    timedatenow = rtime;
  } else {
    time(&timedatenow);
  }
  tminfo = localtime(&timedatenow);
  year = (tminfo -> tm_year) + 1900; /* Jahr        */
  mon = (tminfo -> tm_mon) + 1;      /* Monat       */
  day = (tminfo -> tm_mday);         /* Tag         */
  dow = (tminfo -> tm_wday);         /* Wochentag   */
  stu = (tminfo -> tm_hour);         /* Stunde      */
  min = (tminfo -> tm_min);          /* Minute      */
  sec = (tminfo -> tm_sec);          /* Sekunde     */
  doy = (tminfo -> tm_yday);         /* Tag im Jahr */
  /* Woche */
  if (xday == day) {
    return;
  } else {
    week = getweek(timedatenow);
    xday = day;
  }
}

main(argc, argv)
  int argc;
  char *argv[];
{
  int i;
  struct stat _fstat;

  gettimedate(0);

  // print once ...
  if (argc == 1) {
    printf("%02d:%02d:%02d %02s, %02d/%02d. %03s(%02d) %04d %03d/%02d\n", stu, min, sec, tage[dow], day, mont[mon - 1], mon3[mon - 1], mon, year, doy + 1, week);
    exit(0);
  }

  // help
  if ((strcmp(argv[1], "-?") == 0) || (strcmp(argv[1], "-h") == 0)) {
    printf("bdate [-|-?|-c <file>|<file(s)>]\n\n");
    printf("  -?       : this help\n");
    printf("           : print time/date in Blacky2000 format\n");
    printf("  -        : print time/date in Blacky2000 format continuously\n");
    printf("  <file(s)>: print time/date of file(s) in Blacky2000 format\n");
    printf("  -c <file>: compare file's time/date with time/date in file\n");
    exit(0);
  }



  // compare file's time/date with time/date in file
  if (strcmp(argv[1], "-c") == 0) {
    FILE *f;
    int bufsize;
    char buf[bufsize];
    char tmbuf[50];
    int terror = 0, oerror = 0, anz = 0;
    int len;
    
    if ((f = fopen(argv[2], "r")) == NULL) {
      perror(argv[2]);
      exit(1);
    }
    while (!feof(f)) {
      anz++;
      fgets(buf, bufsize, f);
      len = strlen(buf);
      if (buf[len - 1] == '\n') {
	buf[len - 1] = 0;
      }
      if (stat(buf + 41, &_fstat) == 0) {
	gettimedate(_fstat.st_mtime);
	sprintf(tmbuf, "%02d:%02d:%02d %02s, %02d/%02d. %03s(%02d) %04d %03d/%02d  ", stu, min, sec, tage[dow], day, mont[mon - 1], mon3[mon - 1], mon, year, doy + 1, week);
	if (strncmp(tmbuf, buf, 41) == 0) {
	  printf("%s: OK\n", buf + 41);
	} else {
	  printf("%s: TIME FAIL\n", buf + 41);
	  terror++;
	}
      } else {
	printf("%s: OPEN FAIL\n", buf + 41);
	perror(buf + 41);
	oerror++;
      }
    }
    if ((terror != 0) || (oerror != 0)) {
      printf("FAIL: %03d/%03d Can't read; %03d/%03d Timestamp fail.\n", oerror, anz, terror, anz);
      exit(1);
    }
    exit(0);
  } 

  // print time/date of file(s) in Blacky2000 format
  for (i = 1; i < argc; i++) {
    if (stat(argv[i], &_fstat) == 0) {
      gettimedate(_fstat.st_mtime);
      printf("%02d:%02d:%02d %02s, %02d/%02d. %03s(%02d) %04d %03d/%02d  %s\n", stu, min, sec, tage[dow], day, mont[mon - 1], mon3[mon - 1], mon, year, doy + 1, week, argv[i]);
    } else {
      if (i == 1) {
	goto ENTLOS;
      } else {
	perror(argv[i]);
      }
    }
  }
  exit(0);

  // print continuously ...
  ENTLOS:
  while (1) {
    for (i = 0; i < 60; i++) {
      printf("%c", 8);
    }
    printf("%02d:%02d:%02d %02s, %02d/%02d. %03s(%02d) %04d %03d/%02d", stu, min, sec, tage[dow], day, mont[mon - 1], mon3[mon - 1], mon, year, doy + 1, week);
    fflush(stdout);
    FD_ZERO(&rfds);
    FD_SET(0, &rfds);
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    if (select(1, &rfds, NULL, NULL, &tv) == 1) {
      exit(0);
    }
    gettimedate(0);
  }
}
